"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImageLoaderService = void 0;
const core_1 = require("@angular/core");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const capacitor_filesystem_plugin_1 = require("@spryrocks/capacitor-filesystem-plugin");
const EXTENSIONS = ['jpg', 'png', 'jpeg', 'gif', 'svg', 'tiff'];
let ImageLoaderService = class ImageLoaderService {
    constructor(config, http, platform, webview) {
        this.config = config;
        this.http = http;
        this.platform = platform;
        this.webview = webview;
        /**
         * Indicates if the cache service is ready.
         * When the cache service isn't ready, images are loaded via browser instead.
         */
        this.isCacheReady = false;
        /**
         * Indicates if this service is initialized.
         * This service is initialized once all the setup is done.
         */
        this.isInit = false;
        this.initPromise = new Promise(resolve => this.initPromiseResolve = resolve);
        this.lockSubject = new rxjs_1.Subject();
        this.lock$ = this.lockSubject.asObservable();
        /**
         * Number of concurrent requests allowed
         */
        this.concurrency = 5;
        /**
         * Queue items
         */
        this.queue = [];
        this.processing = 0;
        /**
         * Fast accessible Object for currently processing items
         */
        this.currentlyProcessing = {};
        this.cacheIndex = [];
        this.currentCacheSize = 0;
        this.indexed = false;
        this.lockedCallsQueue = [];
        if (!platform.is('cordova')) {
            // we are running on a browser, or using livereload
            // plugin will not function in this case
            this.isInit = true;
            this.throwWarning('You are running on a browser or using livereload, IonicImageLoader will not function, falling back to browser loading.');
            this.initPromiseResolve();
        }
        else {
            (0, rxjs_1.fromEvent)(document, 'deviceready')
                .pipe((0, operators_1.first)())
                .subscribe(res => {
                if (this.nativeAvailable) {
                    this.initCache();
                }
                else {
                    // we are running on a browser, or using livereload
                    // plugin will not function in this case
                    this.isInit = true;
                    this.initPromiseResolve();
                    this.throwWarning('You are running on a browser or using livereload, IonicImageLoader will not function, falling back to browser loading.');
                }
            });
        }
    }
    get nativeAvailable() {
        return capacitor_filesystem_plugin_1.FileSystem.isAvailable();
    }
    get isCacheSpaceExceeded() {
        return (this.config.maxCacheSize > -1 &&
            this.currentCacheSize > this.config.maxCacheSize);
    }
    get isWKWebView() {
        return (this.platform.is('ios') &&
            window.webkit &&
            window.webkit.messageHandlers);
    }
    get isIonicWKWebView() {
        return (
        //  Important: isWKWebview && isIonicWKWebview must be mutually excluse.
        //  Otherwise the logic for copying to tmp under IOS will fail.
        (this.platform.is('android') && this.webview) ||
            (this.platform.is('android')) && (location.host === 'localhost:8080') ||
            window.LiveReload);
    }
    get isDevServer() {
        return window['IonicDevServer'] !== undefined;
    }
    /**
     * Check if we can process more items in the queue
     */
    get canProcess() {
        return this.queue.length > 0 && this.processing < this.concurrency;
    }
    ready() {
        return this.initPromise;
    }
    /**
     * Preload an image
     * @param imageUrl Image URL
     * @returns returns a promise that resolves with the cached image URL
     */
    preload(imageUrl) {
        return this.getImagePath(imageUrl);
    }
    getFileCacheDirectory() {
        let type;
        if (this.config.cacheDirectoryType === 'data') {
            type = capacitor_filesystem_plugin_1.SystemDirectoryType.Data;
        }
        else if (this.config.cacheDirectoryType === 'external') {
            type = this.platform.is('android') ? capacitor_filesystem_plugin_1.SystemDirectoryType.ExternalData : capacitor_filesystem_plugin_1.SystemDirectoryType.Documents;
        }
        else {
            type = capacitor_filesystem_plugin_1.SystemDirectoryType.Cache;
        }
        return capacitor_filesystem_plugin_1.FileSystem.getSystemDirectory(type);
    }
    /**
     * Clears cache of a single image
     * @param imageUrl Image URL
     */
    clearImageCache(imageUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.platform.is('cordova')) {
                return;
            }
            yield this.ready();
            this.runLocked(() => __awaiter(this, void 0, void 0, function* () {
                const fileName = this.createFileName(imageUrl);
                const route = yield this.getFileCacheDirectory().getDirectory(this.config.cacheDirectoryName);
                // pause any operations
                this.isInit = false;
                try {
                    yield route.getFile(fileName).delete();
                    if (this.isWKWebView && !this.isIonicWKWebView) {
                        yield capacitor_filesystem_plugin_1.FileSystem.getSystemDirectory(capacitor_filesystem_plugin_1.SystemDirectoryType.Temp).getFile(fileName).delete();
                    }
                }
                catch (err) {
                    this.throwError(err);
                }
                return this.initCache(true);
            }));
        });
    }
    /**
     * Clears the cache
     */
    clearCache() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.platform.is('cordova')) {
                return;
            }
            yield this.ready();
            this.runLocked(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.getFileCacheDirectory().getDirectory(this.config.cacheDirectoryName)
                        .delete({ recursively: true });
                    if (this.isWKWebView && !this.isIonicWKWebView) {
                        // also clear the temp files
                        try {
                            yield capacitor_filesystem_plugin_1.FileSystem.getSystemDirectory(capacitor_filesystem_plugin_1.SystemDirectoryType.Temp).getDirectory(this.config.cacheDirectoryName)
                                .delete({ recursively: true });
                        }
                        catch (err) {
                            // Noop catch. Removing the tempDirectory might fail,
                            // as it is not persistent.
                        }
                    }
                }
                catch (err) {
                    this.throwError(err);
                }
                return this.initCache(true);
            }));
        });
    }
    /**
     * Gets the filesystem path of an image.
     * This will return the remote path if anything goes wrong or if the cache service isn't ready yet.
     * @param imageUrl The remote URL of the image
     * @returns Returns a promise that will always resolve with an image URL
     */
    getImagePath(imageUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof imageUrl !== 'string' || imageUrl.length <= 0) {
                throw new Error('The image url provided was empty or invalid.');
            }
            yield this.ready();
            if (!this.isCacheReady) {
                this.throwWarning('The cache system is not running. Images will be loaded by your browser instead.');
                return imageUrl;
            }
            if (this.isImageUrlRelative(imageUrl)) {
                return imageUrl;
            }
            try {
                return yield this.getCachedImagePath(imageUrl);
            }
            catch (err) {
                // image doesn't exist in cache, lets fetch it and save it
                return this.addItemToQueue(imageUrl);
            }
        });
    }
    processLockedQueue() {
        return __awaiter(this, void 0, void 0, function* () {
            if (yield this.getLockedState()) {
                return;
            }
            if (this.lockedCallsQueue.length > 0) {
                yield this.setLockedState(true);
                try {
                    yield this.lockedCallsQueue.slice(0, 1)[0]();
                }
                catch (err) {
                    console.log('Error running locked function: ', err);
                }
                yield this.setLockedState(false);
                return this.processLockedQueue();
            }
        });
    }
    getLockedState() {
        return this.lock$
            .pipe((0, operators_1.take)(1))
            .toPromise();
    }
    awaitUnlocked() {
        return this.lock$
            .pipe((0, operators_1.filter)(locked => !!locked), (0, operators_1.take)(1))
            .toPromise();
    }
    setLockedState(locked) {
        return __awaiter(this, void 0, void 0, function* () {
            this.lockSubject.next(locked);
        });
    }
    runLocked(fn) {
        this.lockedCallsQueue.push(fn);
        this.processLockedQueue();
    }
    /**
     * Returns if an imageUrl is an relative path
     * @param imageUrl
     */
    isImageUrlRelative(imageUrl) {
        return !/^(https?|file):\/\/\/?/i.test(imageUrl);
    }
    /**
     * Add an item to the queue
     * @param imageUrl
     * @param resolve
     * @param reject
     */
    addItemToQueue(imageUrl, resolve, reject) {
        let p;
        if (!resolve && !reject) {
            p = new Promise((res, rej) => {
                resolve = res;
                reject = rej;
            });
        }
        else {
            resolve = resolve || (() => {
            });
            reject = reject || (() => {
            });
        }
        this.queue.push({
            imageUrl,
            resolve,
            reject,
        });
        this.processQueue();
        return p;
    }
    /**
     * Processes one item from the queue
     */
    processQueue() {
        return __awaiter(this, void 0, void 0, function* () {
            // make sure we can process items first
            if (!this.canProcess) {
                return;
            }
            // increase the processing number
            this.processing++;
            // take the first item from queue
            const currentItem = this.queue.splice(0, 1)[0];
            // function to call when done processing this item
            // this will reduce the processing number
            // then will execute this function again to process any remaining items
            const done = () => {
                this.processing--;
                this.processQueue();
                // only delete if it's the last/unique occurrence in the queue
                if (this.currentlyProcessing[currentItem.imageUrl] !== undefined && !this.currentlyInQueue(currentItem.imageUrl)) {
                    delete this.currentlyProcessing[currentItem.imageUrl];
                }
            };
            const error = (e) => {
                currentItem.reject();
                this.throwError(e);
                done();
            };
            if (this.currentlyProcessing[currentItem.imageUrl] !== undefined) {
                try {
                    // Prevented same Image from loading at the same time
                    yield this.currentlyProcessing[currentItem.imageUrl];
                    const localUrl = yield this.getCachedImagePath(currentItem.imageUrl);
                    currentItem.resolve(localUrl);
                    done();
                }
                catch (err) {
                    error(err);
                }
                return;
            }
            this.currentlyProcessing[currentItem.imageUrl] = (() => __awaiter(this, void 0, void 0, function* () {
                // process more items concurrently if we can
                if (this.canProcess) {
                    this.processQueue();
                }
                const localDir = this.getFileCacheDirectory().getDirectory(this.config.cacheDirectoryName);
                const fileName = this.createFileName(currentItem.imageUrl);
                try {
                    const data = yield this.http.get(currentItem.imageUrl, {
                        responseType: 'blob',
                        headers: this.config.httpHeaders,
                    }).toPromise();
                    const file = localDir.getFile(fileName);
                    const writer = yield file.openWriter({ replace: true });
                    yield writer.writeBlob(data);
                    if (this.isCacheSpaceExceeded) {
                        this.maintainCacheSize();
                    }
                    yield this.addFileToIndex(file);
                    const localUrl = yield this.getCachedImagePath(currentItem.imageUrl);
                    currentItem.resolve(localUrl);
                    done();
                    this.maintainCacheSize();
                }
                catch (err) {
                    error(err);
                    throw err;
                }
            }))();
        });
    }
    /**
     * Search if the url is currently in the queue
     * @param imageUrl Image url to search
     */
    currentlyInQueue(imageUrl) {
        return this.queue.some(item => item.imageUrl === imageUrl);
    }
    /**
     * Initialize the cache service
     * @param [replace] Whether to replace the cache directory if it already exists
     */
    initCache(replace) {
        return __awaiter(this, void 0, void 0, function* () {
            this.concurrency = this.config.concurrency;
            // create cache directories if they do not exist
            try {
                yield this.createCacheDirectory(replace);
                yield this.indexCache();
                this.isCacheReady = true;
            }
            catch (err) {
                this.throwError(err);
            }
            this.isInit = true;
            this.initPromiseResolve();
        });
    }
    /**
     * Adds a file to index.
     * Also deletes any files if they are older than the set maximum cache age.
     * @param file FileEntry to index
     */
    addFileToIndex(file) {
        return __awaiter(this, void 0, void 0, function* () {
            const metadata = yield file.getMetadata();
            if (this.config.maxCacheAge > -1 &&
                Date.now() - metadata.modificationTime.getTime() >
                    this.config.maxCacheAge) {
                // file age exceeds maximum cache age
                return this.removeFile(file.name);
            }
            else {
                // file age doesn't exceed maximum cache age, or maximum cache age isn't set
                this.currentCacheSize += metadata.size;
                // add item to index
                this.cacheIndex.push({
                    name: file.name,
                    modificationTime: metadata.modificationTime,
                    size: metadata.size,
                });
            }
        });
    }
    /**
     * Indexes the cache if necessary
     */
    indexCache() {
        return __awaiter(this, void 0, void 0, function* () {
            this.cacheIndex = [];
            try {
                const files = yield this.getFileCacheDirectory().getDirectory(this.config.cacheDirectoryName).getFiles();
                yield Promise.all(files.map(this.addFileToIndex.bind(this)));
                // Sort items by date. Most recent to oldest.
                this.cacheIndex = this.cacheIndex.sort((a, b) => (a > b ? -1 : a < b ? 1 : 0));
                this.indexed = true;
            }
            catch (err) {
                this.throwError(err);
            }
        });
    }
    /**
     * This method runs every time a new file is added.
     * It checks the cache size and ensures that it doesn't exceed the maximum cache size set in the config.
     * If the limit is reached, it will delete old images to create free space.
     */
    maintainCacheSize() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.config.maxCacheSize > -1 && this.indexed) {
                const maintain = () => __awaiter(this, void 0, void 0, function* () {
                    if (this.currentCacheSize > this.config.maxCacheSize) {
                        // grab the first item in index since it's the oldest one
                        const file = this.cacheIndex.splice(0, 1)[0];
                        if (typeof file === 'undefined') {
                            return maintain();
                        }
                        // delete the file then process next file if necessary
                        try {
                            yield this.removeFile(file.name);
                        }
                        catch (err) {
                            // ignore errors, nothing we can do about it
                        }
                        this.currentCacheSize -= file.size;
                        return maintain();
                    }
                });
                return maintain();
            }
        });
    }
    /**
     * Remove a file
     * @param file The name of the file to remove
     */
    removeFile(file) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.getFileCacheDirectory()
                .getDirectory(this.config.cacheDirectoryName)
                .getFile(file)
                .delete();
            if (this.isWKWebView && !this.isIonicWKWebView) {
                try {
                    yield capacitor_filesystem_plugin_1.FileSystem.getSystemDirectory(capacitor_filesystem_plugin_1.SystemDirectoryType.Temp)
                        .getDirectory(this.config.cacheDirectoryName)
                        .getFile(file)
                        .delete();
                }
                catch (err) {
                    // Noop catch. Removing the files from tempDirectory might fail, as it is not persistent.
                }
            }
        });
    }
    /**
     * Get the local path of a previously cached image if exists
     * @param url The remote URL of the image
     * @returns Returns a promise that resolves with the local path if exists, or rejects if doesn't exist
     */
    getCachedImagePath(url) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.ready();
            if (!this.isCacheReady) {
                throw new Error('Cache is not ready');
            }
            // if we're running with livereload, ignore cache and call the resource from it's URL
            if (this.isDevServer) {
                return url;
            }
            // get file name
            const fileName = this.createFileName(url);
            // get full path
            const dir = this.getFileCacheDirectory().getDirectory(this.config.cacheDirectoryName);
            const tempDir = capacitor_filesystem_plugin_1.FileSystem.getSystemDirectory(capacitor_filesystem_plugin_1.SystemDirectoryType.Temp).getDirectory(this.config.cacheDirectoryName);
            try {
                // check if exists
                const fileEntry = yield this.resolveLocalFile(dir, fileName);
                // file exists in cache
                if (this.config.imageReturnType === 'base64') {
                    // read the file as data url and return the base64 string.
                    // should always be successful as the existence of the file
                    // is already ensured
                    const base64 = yield this.readFileAsDataURL(fileEntry);
                    return base64.replace('data:null', 'data:*/*');
                }
                else if (this.config.imageReturnType !== 'uri') {
                    return;
                }
                // now check if iOS device & using WKWebView Engine.
                // in this case only the tempDirectory is accessible,
                // therefore the file needs to be copied into that directory first!
                if (this.isIonicWKWebView) {
                    return this.normalizeUrl(fileEntry);
                }
                if (!this.isWKWebView) {
                    // return native path
                    return fileEntry.url;
                }
                // check if file already exists in temp directory
                try {
                    const tempFileEntry = yield this.resolveLocalFile(tempDir, fileName);
                    // file exists in temp directory
                    // return native path
                    return this.normalizeUrl(tempFileEntry);
                }
                catch (err) {
                    // file does not yet exist in the temp directory.
                    // copy it!
                    const tempFileEntry = tempDir.getFile(fileName);
                    yield dir.getFile(fileName).copyTo(tempFileEntry);
                    // now the file exists in the temp directory
                    // return native path
                    return this.normalizeUrl(tempFileEntry);
                }
            }
            catch (err) {
                throw new Error('File does not exist');
            }
        });
    }
    resolveLocalFile(dir, fileName) {
        return __awaiter(this, void 0, void 0, function* () {
            const file = dir.getFile(fileName);
            if (!(yield file.exists())) {
                throw new Error("File not exists");
            }
            return file;
        });
    }
    /**
     * Normalizes the image uri to a version that can be loaded in the webview
     * @param fileEntry the FileEntry of the image file
     * @returns the normalized Url
     */
    normalizeUrl(fileEntry) {
        // Use Ionic normalizeUrl to generate the right URL for Ionic WKWebView
        if (Ionic && typeof Ionic.normalizeURL === 'function') {
            return Ionic.normalizeURL(fileEntry.url);
        }
        // use new webview function to do the trick
        if (this.webview) {
            return this.webview.convertFileSrc(fileEntry.url);
        }
        return fileEntry.url;
    }
    /**
     * Throws a console error if debug mode is enabled
     * @param args Error message
     */
    throwError(...args) {
        if (this.config.debugMode) {
            args.unshift('ImageLoader Error: ');
            console.error.apply(console, args);
        }
    }
    /**
     * Throws a console warning if debug mode is enabled
     * @param args Error message
     */
    throwWarning(...args) {
        if (this.config.debugMode) {
            args.unshift('ImageLoader Warning: ');
            console.warn.apply(console, args);
        }
    }
    /**
     * Check if the cache directory exists
     * @param directory The directory to check. Either this.file.tempDirectory or this.getFileCacheDirectory()
     * @returns Returns a promise that resolves if exists, and rejects if it doesn't
     */
    directoryExistsOrThrow(directory) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!(directory.exists()))
                throw new Error("Directory not exists");
        });
    }
    /**
     * Create the cache directories
     * @param replace override directory if exists
     * @returns Returns a promise that resolves if the directories were created, and rejects on error
     */
    createCacheDirectory(replace = false) {
        let cacheDirectoryPromise, tempDirectoryPromise;
        const cacheDir = this.getFileCacheDirectory().getDirectory(this.config.cacheDirectoryName);
        if (replace) {
            // create or replace the cache directory
            cacheDirectoryPromise = cacheDir.create({ replace: replace });
        }
        else {
            // check if the cache directory exists.
            // if it does not exist create it!
            cacheDirectoryPromise = this.directoryExistsOrThrow(cacheDir)
                .catch(() => cacheDir.create({ replace: false }));
        }
        if (this.isWKWebView && !this.isIonicWKWebView) {
            const tempDir = capacitor_filesystem_plugin_1.FileSystem.getSystemDirectory(capacitor_filesystem_plugin_1.SystemDirectoryType.Temp).getDirectory(this.config.cacheDirectoryName);
            if (replace) {
                // create or replace the temp directory
                tempDirectoryPromise = tempDir.create({ replace: replace });
            }
            else {
                // check if the temp directory exists.
                // if it does not exist create it!
                tempDirectoryPromise = this.directoryExistsOrThrow(tempDir)
                    .catch(() => tempDir.create({ replace: false }));
            }
        }
        else {
            tempDirectoryPromise = Promise.resolve();
        }
        return Promise.all([cacheDirectoryPromise, tempDirectoryPromise]);
    }
    /**
     * Creates a unique file name out of the URL
     * @param url URL of the file
     * @returns Unique file name
     */
    createFileName(url) {
        // hash the url to get a unique file name
        return (this.hashString(url).toString() +
            (this.config.fileNameCachedWithExtension
                ? this.getExtensionFromUrl(url)
                : ''));
    }
    /**
     * Converts a string to a unique 32-bit int
     * @param string string to hash
     * @returns 32-bit int
     */
    hashString(string) {
        let hash = 0, char;
        if (string.length === 0) {
            return hash;
        }
        for (let i = 0; i < string.length; i++) {
            char = string.charCodeAt(i);
            // tslint:disable-next-line
            hash = (hash << 5) - hash + char;
            // tslint:disable-next-line
            hash = hash & hash;
        }
        return hash;
    }
    /**
     * Extract extension from filename or url
     *
     * @param url
     * @returns
     *
     * Not always will url's contain a valid image extention. We'll check if any valid extention is supplied.
     * If not, we will use the default.
     */
    getExtensionFromUrl(url) {
        const urlWitoutParams = url.split(/\#|\?/)[0];
        const ext = (urlWitoutParams.substr((~-urlWitoutParams.lastIndexOf('.') >>> 0) + 1) || '').toLowerCase();
        return (EXTENSIONS.indexOf(ext) >= 0 ? ext : this.config.fallbackFileNameCachedExtension);
    }
    readFileAsDataURL(file) {
        return __awaiter(this, void 0, void 0, function* () {
            const reader = yield file.openReader();
            return reader.readAsDataUrl();
        });
    }
};
exports.ImageLoaderService = ImageLoaderService;
exports.ImageLoaderService = ImageLoaderService = __decorate([
    (0, core_1.Injectable)({
        providedIn: 'root',
    })
], ImageLoaderService);
//# sourceMappingURL=image-loader.service.js.map